package net.afyre.haha.demo.haha;

import org.eclipse.emf.ecore.util.EcoreUtil;

import net.afyre.haha.demo.haha.ArrayType;
import net.afyre.haha.demo.haha.HahaFactory;
import net.afyre.haha.demo.haha.PrimitiveType;
import net.afyre.haha.demo.haha.SimpleType;
import net.afyre.haha.demo.haha.Type;
import net.afyre.haha.demo.haha.util.HahaSwitch;

public final class TypeUtils {
  
  /**
   * The size in bits of the @link{PrimitiveType.INT} type.
   */
  public static int INT_SIZE = 32;
  /**
   * The size in bits of the @link{PrimitiveType.BOOLEAN} type.
   */
  public static int BOOLEAN_SIZE = 1;
  
  private static TypeUtils instance = new TypeUtils();
  
  private final HahaFactory factory;
  private final SimpleType booleanType;
  private final SimpleType intType;
  private final SimpleType zType;
  
  
  public static TypeUtils getInstance() {
    return instance;
  }
  
  private TypeUtils() {
    factory = HahaFactory.eINSTANCE;
    booleanType = factory.createSimpleType();
    booleanType.setType(PrimitiveType.BOOLEAN);
    intType = factory.createSimpleType();
    intType.setType(PrimitiveType.INT);
    zType = factory.createSimpleType();
    zType.setType(PrimitiveType.Z);
  }
  
  public SimpleType getBoolean() {
    return booleanType;
  }
  
  public SimpleType getInt() {
    return intType;
  }
  
  public SimpleType getZ() {
    return zType;
  }
  
  public ArrayType getArray(final Type elements) {
    final ArrayType result = factory.createArrayType();
    result.setElementType(EcoreUtil.copy(elements));
    return result;
  }
  
  public boolean typesAreEqual(final Type type1, final Type type2) {
    return new EqualsSwitch(type2).doSwitch(type1);
  }
}

class SimpleTypeEquals extends HahaSwitch<Boolean> {
  private final PrimitiveType expectedType;
  
  public SimpleTypeEquals(final SimpleType type) {
    expectedType = type.getType();
  }
  
  @Override
  public Boolean caseType(final Type type) {
    return false;
  }
  
  @Override
  public Boolean caseSimpleType(final SimpleType other) {
    return other != null && other.getType() == expectedType;
  }
}

class ArrayTypeEquals extends HahaSwitch<Boolean> {
  private final Type elementType;

  public ArrayTypeEquals(final ArrayType type) {
    this.elementType = type.getElementType();
  }

  @Override
  public Boolean caseType(final Type type) {
    return false;
  }
  
  @Override
  public Boolean caseArrayType(final ArrayType other) {
    if (other == null) { return false; }
    final EqualsSwitch eq = new EqualsSwitch(elementType);
    return eq.doSwitch(other.getElementType());
  }
}

class EqualsSwitch extends HahaSwitch<Boolean> {
  private final Type secondType;

  public EqualsSwitch(Type secondType) {
    this.secondType = secondType;
  }
  
  @Override
  public Boolean caseSimpleType(final SimpleType type) {
    return new SimpleTypeEquals(type).doSwitch(secondType);
  }
  
  @Override
  public Boolean caseArrayType(final ArrayType type) {
    return new ArrayTypeEquals(type).doSwitch(secondType);
  }
}
