package net.afyre.haha.demo.rcp.handlers;

import java.io.File;
import java.io.IOException;
import java.net.URI;

import org.eclipse.core.filesystem.EFS;
import org.eclipse.core.filesystem.IFileStore;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IPersistableElement;
import org.eclipse.ui.IURIEditorInput;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.editors.text.ILocationProvider;

/**
 * Editor input used by the 'New file' command.
 * 
 * Based on UntitledTextFileInput.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class NewFileInput implements IEditorInput, IURIEditorInput, ILocationProvider {
  private static int counter = 0;
  
  private final IFileStore store;
  private final String name;
  
  public NewFileInput() throws IOException {  
    final File file;
    
    file = File.createTempFile("__haha_", ".haha");
    file.deleteOnExit();
    final IPath path = new Path(file.getAbsolutePath());
    store = EFS.getLocalFileSystem().getStore(path);
    counter++;
    name = "New file " + counter;
  }

  /** {@inheritDoc} */
  @SuppressWarnings({ "rawtypes", "unchecked" })
  @Override
  public Object getAdapter(final Class adapter) {
    if (ILocationProvider.class.equals(adapter)) {
      return this;
    } else {
      return Platform.getAdapterManager().getAdapter(this, adapter);
    }
  }

  /** {@inheritDoc} */
  @Override
  public IPath getPath(final Object element) {
    if (element instanceof NewFileInput) {
      final NewFileInput input = (NewFileInput)element;
      return new Path(input.store.toURI().getPath());
    } else {
      return null;
    }
  }

  /** {@inheritDoc} */
  @Override
  public boolean exists() {
    return false;
  }

  /** {@inheritDoc} */
  @Override
  public ImageDescriptor getImageDescriptor() {
    return PlatformUI.getWorkbench().getEditorRegistry().getImageDescriptor(name);
  }

  /** {@inheritDoc} */
  @Override
  public String getName() {
    return name;
  }

  /** {@inheritDoc} */
  @Override
  public IPersistableElement getPersistable() {
    return null;
  }

  /** {@inheritDoc} */
  @Override
  public String getToolTipText() {
    return name;
  }

  /** {@inheritDoc} */
  @Override
  public URI getURI() {
    return store.toURI();
  }
}
