package net.afyre.haha.demo.solver.smt;

/**
 * CVC4 solver.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class CVC4 extends SmtSolver {
  /** Command line arguments. */
  private static final String[] CMD_ARGS = new String[] {
      "--lang", "smt2", // Use SMT2 input language 
      "--incremental", // Support push/op.
  };
  
  /**
   * Constructor.
   * @param solverPath Path to the Z3 executable.
   * @param logger Object used to print log messages.
   */
  protected CVC4(final String solverPath,
                 final SmtLogger logger) {
    super(solverPath, CMD_ARGS, logger);
  }
  
  /**
   * Create a solver.
   * @param solverPath Path to the Z3 executable.
   * @param logger Object used to print log messages.
   * @return Solver instance.
   */
  public static CVC4 getInstance(final String solverPath,
                                 final SmtLogger logger) {
    return new CVC4(solverPath, logger);
  }
  
  /** {@inheritDoc} */
  @Override
  protected void sendPreamble() throws SmtSolver.CancellationException {
    send(list("set-logic", "ALL_SUPPORTED"));
    sendOption("produce-models", "true");
    //sendOption("soft-timeout", "10000");
    mark();
  }
}
