package net.afyre.haha.demo.solver.smt;

/**
 * Interface of objects used to report errors and warnings.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public interface SmtLogger {
  /**
   * Report an error.
   * @param msg Error message.
   */
  void error(String msg);

  /**
   * Report an error.
   * @param msg Error message.
   * @param cause Exception which caused the error.
   */
  void error(String msg, Throwable cause);
  
  /**
   * Report a model (counterexample) provided by the solver.
   * @param model Model.
   */
  void model(SmtModel model);
  
  /**
   * Report a warning.
   * @param msg Warning message.
   */
  void warning(String msg);

  /**
   * Report a warning.
   * @param msg Warning message.
   * @param cause Exception which caused the warning.
   */
  void warning(String msg, Throwable cause);
  
  /**
   * Report some debug information.
   * @param msg Message.
   */
  void info(String msg);
  
  /**
   * Report raw input sent to the solver.
   * @param text Text sent to the solver.
   */
  void input(String text);

  /**
   * Report raw output received from the solver.
   * @param text Text received from the solver.
   */
  void output(String text);
}
