package net.afyre.haha.demo.solver.smt;

import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.Status;
import org.osgi.framework.BundleContext;

/**
 * Plugin class. 
 *
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class SmtPlugin extends Plugin {
  /** Last created instance.  */
  private static SmtPlugin instance;
  
  /** Context handle. */
  private BundleContext context;
  
  /** Plugin ID. */
  private String id;
  
  /** Constructor. */
  public SmtPlugin() {
    instance = this;
  }
  
  /**
   * Get the last created instance. 
   * @return Instance or null (if no instance was created)
   */
  public static SmtPlugin getInstance() {
    return instance;
  }
  
  /** {@inheritDoc} */
  @Override
  public void start(final BundleContext bundleContext) throws Exception {
    context = bundleContext;
    id = context.getBundle().getSymbolicName();
  }

  /** {@inheritDoc} */
  @Override
  public void stop(final BundleContext bundleContext) throws Exception {
    context = null;
  }

  /** 
   * Get ID of this plugin.
   * 
   * @return Plugin ID.
   */
  public String getId() {
    return id;
  }
  
  /** Wrapper for error messages produced by this plugin. */
  public class SmtError extends Status {
    /**
     * Create a new error message.
     * @param msg Message body.
     */
    public SmtError(final String msg) {
      super(ERROR, getId(), msg);
    }
    
     /**
     * Create a new error message.
     * @param msg Message body.
     * @param exc Error cause.
     */
    public SmtError(final String msg, final Throwable exc) {
      super(ERROR, getId(), msg, exc);
    }
    
  }

  /** Wrapper for warning messages produced by this plugin. */
  public class SmtWarning extends Status {
    /**
     * Create a new warning message.
     * @param msg Message body.
     */
    public SmtWarning(final String msg) {
      super(WARNING, getId(), msg);
    }
    
    /**
    * Create a new warning message.
    * @param msg Message body.
    * @param exc Problem cause.
    */
    public SmtWarning(final String msg, final Throwable exc) {
      super(WARNING, getId(), msg, exc);
    }
  }
  
  /** Wrapper for info messages produced by this plugin. */
  public class SmtInfo extends Status {
    /**
    * Create a new info message.
    * @param msg Message body.
    */
    public SmtInfo(final String msg) {
      super(INFO, getId(), msg);
    }
  }
  
  /** Wrapper for messages which describe a SMT model. */
  public class SmtModelInfo extends Status {
    /**
     * Create new message.
     * @param model SMT model.
     */
    public SmtModelInfo(final SmtModel model) {
      super(INFO, getId(), "Model: " +  model.toString());
    }
  }
}
