package net.afyre.haha.demo.solver.smt;

import org.eclipse.core.runtime.ILog;

/**
 * A logger implementation which uses the eclipse error log.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class SmtPluginLogger implements SmtLogger {
  /** Eclipse logger. */
  private final ILog logger;
 
  /** Plugin handle. */
  private final SmtPlugin plugin;
  
  /** Constructor. */
  public SmtPluginLogger() {
    plugin = SmtPlugin.getInstance();
    logger = plugin.getLog();
  }
  
  /** {@inheritDoc} */
  @Override
  public void error(final String msg) { 
    logger.log(plugin.new SmtError(msg));
  }

  /** {@inheritDoc} */
  @Override
  public void error(final String msg, 
                    final Throwable cause) { 
    logger.log(plugin.new SmtError(msg, cause));
  }

  /** {@inheritDoc} */
  @Override
  public void model(final SmtModel model) {
    logger.log(plugin.new SmtModelInfo(model));
  }
  
  /** {@inheritDoc} */
  @Override
  public void warning(final String msg) {
    logger.log(plugin.new SmtWarning(msg));
  }

  /** {@inheritDoc} */
  @Override
  public void warning(final String msg, 
                      final Throwable cause) {
    logger.log(plugin.new SmtWarning(msg, cause));
  }

  /** {@inheritDoc} */
  @Override
  public void info(final String msg) {
    logger.log(plugin.new SmtInfo(msg));
  }
  
  /** Solver output is ignored. */
  
  /** {@inheritDoc} */
  @Override
  public void input(final String text) { }
  
  /** {@inheritDoc} */
  @Override
  public void output(final String text) { }
}
