package net.afyre.haha.demo.solver.smt;

/**
 * Z3 solver.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class Z3 extends SmtSolver {
  /** Command line arguments. */
  private static final String[] CMD_ARGS = new String[] {
      "-smt2", // Use SMT2 input language 
      "-in" // read from standard input
  };
  
  /** Z3 solver version */
  private String version;
  
  /**
   * Constructor.
   * @param solverPath Path to the Z3 executable.
   * @param solverVersion Version of the Z3 in the executable. 
   * @param logger Object used to print log messages.
   */
  protected Z3(final String solverPath,
               final String solverVersion,
               final SmtLogger logger) {
    super(solverPath, CMD_ARGS, logger);
    this.version = solverVersion; 
  }
  
  /**
   * Create a solver.
   * @param solverPath Path to the Z3 executable.
   * @param solverVersion Version of the Z3 in the executable.
   * @param logger Object used to print log messages.
   * @return Solver instance.
   */
  public static Z3 getInstance(final String solverPath,
                               final String solverVersion,
                               final SmtLogger logger) {
    return new Z3(solverPath, solverVersion, logger);
  }
  
  /** {@inheritDoc} */
  @Override
  protected void sendPreamble() throws SmtSolver.CancellationException {
    sendOption("produce-models", "true");
    sendOption("produce-proofs", "false");
    if (version.compareTo("4.3.0") == 0) {
      sendOption("soft-timeout", "10000");
      sendOption("mbqi", "true");
    } else {
      sendOption("timeout", "10000");
      sendOption("smt.mbqi", "true");
    }
  }
}
