package net.afyre.haha.demo.solver;

import java.util.HashSet;
import java.util.Iterator;

/**
 * The class gathers global state of running solvers. In particular the
 * processes of the solvers.
 * 
 * @author Aleksy Schubert <alx@mimuw.edu.pl>
 *
 */
public final class SolverRunningState {
  
  private static HashSet<Process> verificationProcesses = 
      new HashSet<Process>();

  /**
   * The construct to prevent creation of instances.
   */
  private SolverRunningState() {
    
  }
  
  /**
   * The method kills all currently active verification processes.
   * 
   */
  public static synchronized void killActiveVerificationProcesses() {
    for (final Iterator<Process> i = verificationProcesses.iterator(); i
        .hasNext();) {
      final Process process = i.next();
      process.destroy();
      System.out.println(process.toString());
      verificationProcesses.remove(process);
    }
  }

  /**
   * This method adds a process to the group of active verification processes.
   * 
   * @param pr the process to add
   */
  public static synchronized void addActiveVerificationProcess(
      final Process pr) {
    verificationProcesses.add(pr);
  }

  /**
   * This method removes a process from the group of active verification 
   * processes.
   * 
   * @param pr the process to remove
   */
  public static synchronized void rmActiveVerificationProcess(
       final Process pr) {
    verificationProcesses.remove(pr);
  }
}
