package net.afyre.haha.demo.solver.context;

import net.afyre.haha.demo.solver.expr.formulae.Formula;

/**
 * Solver context handle.
 * 
 * Context handles are used to communicate with solvers.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public interface SolverContext {
  /**
   * Add a declaration to the current context.
   * @param declaration Declaration.
   * @return True on success, false on error.
   */
  boolean addDeclaration(Declaration declaration);

  /**
   * Mark current solver state.
   * @see #reset()
   * @return True on success, false on error.
   */
  boolean mark();
  
  /**
   * Undo all declarations made after last mark,
   * and remove that mark. If there are no marked
   * states, all declarations are removed.
   * @see #mark()
   * @return True on success, false on error.
   */
  boolean reset();
  
  /**
   * Check validity of a formula.
   * @param query Formula to be checked.
   * @return Solver answer.
   */
  Answer isValid(Formula query);
  
  /**
   * Check satisfiability of a formula.
   * @param query Formula to be checked.
   * @return Solver answer.
   */
  Answer isSatisfiable(Formula query);
  
  /**
   * Release all external resources used
   * by this solver. 
   */
  void finish();
}
