package net.afyre.haha.demo.solver.context;

import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Declaration or definition of a value.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class TermDeclaration extends AbstractDeclaration {
  /** Type. */
  private final ValueType type;
  
  /** Value definition (null in case of a declaration). */
  private final Term value;

  /**
   * Create a term declaration.
   * @param name Variable name.
   * @param type Value type.
   */
  public TermDeclaration(final String name,
                         final ValueType type) {
    super(name);
    this.type = type;
    this.value = null;
  }
  
  /**
   * Create a term definition.
   * @param name Variable name.
   * @param type Value type.
   * @param value Value definition (null for declarations).
   */
  public TermDeclaration(final String name,
                         final ValueType type,
                         final Term value) {
    super(name);
    this.type = type;
    this.value = value;
  }

  /** {@inheritDoc} */
  @Override
  public void accept(final DeclarationVisitor visitor) {
    if (value == null) {
      visitor.visitTermDeclaration(getName(), type);
    } else {
      visitor.visitTermDefinition(getName(), type, value);
    }
  }
  
}
