package net.afyre.haha.demo.solver.expr.formulae.impl;

import net.afyre.haha.demo.solver.expr.formulae.Formula;
import net.afyre.haha.demo.solver.expr.formulae.FormulaSwitch;

/**
 * Local propositional variable binding.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class FLetProp extends AbstractFormula {
  /** Variable name. */
  private final String var;
  
  /** Value expression. */
  private final Formula formula;
  
  /** Expression body. */
  private final Formula body;

  /**
   * Constructor.
   * @param var Variable name.
   * @param formula Assigned value.
   * @param body Expression body.
   */
  public FLetProp(final String var, 
                 final Formula formula, 
                 final Formula body) {
    this.var = var;
    this.formula = formula;
    this.body = body;
  }

  /**
   * Get the assigned variable.
   * @return Variable.
   */
  public String getVar() {
    return var;
  }

  /**
   * Get the assigned value.
   * @return Value.
   */
  public Formula getFormula() {
    return formula;
  }

  /**
   * Get the expression body.
   * @return Expression.
   */
  public Formula getBody() {
    return body;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final FormulaSwitch<? extends Result> v) {
    return v.caseFLetProp(var, formula, body);
  }
  
  
}
