package net.afyre.haha.demo.solver.expr.terms.impl;

import net.afyre.haha.demo.solver.expr.terms.Term;
import net.afyre.haha.demo.solver.expr.terms.TermSwitch;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Local variable binding.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class Let extends AbstractTerm {
  /** Variable name. */
  private final String var;
  
  /** Type of the bound variable. */
  private final ValueType type;
  
  /** Value expression. */
  private final Term value;
  
  /** Expression body. */
  private final Term body;

  /**
   * Constructor.
   * @param var Variable name.
   * @param type Variable type.
   * @param value Assigned value.
   * @param body Expression body.
   */
  public Let(final String var, 
             final ValueType type,
             final Term value, 
             final Term body) {
    this.var = var;
    this.type = type;
    this.value = value;
    this.body = body;
  }

  /**
   * Get the assigned variable.
   * @return Variable.
   */
  public String getVar() {
    return var;
  }

  /**
   * Get type of the bound variable.
   * @return Type
   */
  public ValueType getType() {
    return type;
  }
  
  /**
   * Get the assigned value.
   * @return Value.
   */
  public Term getValue() {
    return value;
  }

  /**
   * Get the expression body.
   * @return Expression.
   */
  public Term getBody() {
    return body;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final TermSwitch<? extends Result> v) {
    return v.caseLet(var, type, value, body);
  }
}
