package net.afyre.haha.demo.solver.expr.types.impl;

import java.util.Arrays;

import net.afyre.haha.demo.solver.expr.types.NonValueTypeSwitch;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Functions.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class Function extends AbstractNonValueType {
  /** Result type. */
  private final ValueType resultType;
  
  /** Types of arguments. */
  private final ValueType[] argTypes;

  /**
   * Constructor.
   * @param resultType Type of the result.
   * @param argTypes Types of the arguments.
   */
  public Function(final ValueType resultType, 
                  final ValueType[] argTypes) {
    this.resultType = resultType;
    this.argTypes = Arrays.copyOf(argTypes, argTypes.length);
  }

  /**
   * Get the result type.
   * @return Result type.
   */
  public ValueType getResultType() {
    return resultType;
  }

  /**
   * Get types of the arguments.
   * @return List of types.
   */
  public ValueType[] getArgTypes() {
    return argTypes;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final NonValueTypeSwitch<? extends Result> v) {
    return v.caseFunction(resultType, argTypes);
  }
}
