package net.afyre.haha.demo.solver.expr.types.impl;

import java.util.Arrays;

import net.afyre.haha.demo.solver.expr.types.NonValueTypeSwitch;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Predicates (functions with boolean codomain).
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class Predicate extends AbstractNonValueType {
  /** Types of arguments. */
  private final ValueType[] argTypes;

  /**
   * Constructor.
   * @param argTypes Types of the arguments.
   */
  public Predicate(final ValueType[] argTypes) {
    this.argTypes = Arrays.copyOf(argTypes, argTypes.length);
  }

  /**
   * Get types of the arguments.
   * @return List of types.
   */
  public ValueType[] getArgTypes() {
    return argTypes;
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final NonValueTypeSwitch<? extends Result> v) {
    return v.casePredicate(argTypes);
  }
}
