package net.afyre.haha.demo.solver.expr.types.impl;

import java.util.Arrays;

import net.afyre.haha.demo.solver.expr.types.StructuredTypeSwitch;
import net.afyre.haha.demo.solver.expr.types.ValueType;

/**
 * Tuples.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public final class Tuple extends AbstractStructuredType {
  /** Types of the elements. */
  private final ValueType[] types;
  
  /**
   * Constructor.
   * @param types Types of the elements.
   */
  public Tuple(final ValueType... types) {
    this.types = Arrays.copyOf(types, types.length);
  }
  
  /**
   * Get types of the elements.
   * @return List of types.
   */
  public ValueType[] getTypes() {
    return Arrays.copyOf(types, types.length);
  }

  /** {@inheritDoc} */
  @Override
  public<Result> Result accept(final StructuredTypeSwitch<? extends Result> v) {
    return v.caseTuple(getTypes());
  }
}
