package net.afyre.haha.demo.tests;

import net.afyre.haha.demo.testsutils.HahaTestsInjectorProvider;

import org.eclipse.xtext.junit4.InjectWith;
import org.eclipselabs.xtext.utils.unittesting.XtextRunner2;
import org.eclipselabs.xtext.utils.unittesting.XtextTest;
import org.junit.Test;
import org.junit.runner.RunWith;

@InjectWith(HahaTestsInjectorProvider.class)
@RunWith(XtextRunner2.class)

/**
 * Based upon http://code.google.com/a/eclipselabs.org/p/xtext-utils/wiki/Unit_Testing
 * and http://svn.codespot.com/a/eclipselabs.org/xtext-utils/subprojects/unittesting/trunk/examples/
 * 
 * @author Aleksy Schubert
 *
 */
public class HahaParserTesting extends XtextTest {

  public HahaParserTesting() {
    super("HahaParserTesting");
  }

  @Test
  public void id(){
    testTerminal("bar", /* token stream: */ "ID");
    testTerminal("bar3", /* token stream: */ "ID");
    testTerminal("_bar_", /* token stream: */ "ID");
    testTerminal("^bar_", /* token stream: */ "ID");

    testNotTerminal("3bar", /* unexpected */ "ID");
    testNotTerminal("#bar", /* unexpected */ "ID");
  }
  
  @Test
  public void comments(){
    testTerminal("/* sth */", /* token stream: */ "ML_COMMENT");
    testTerminal("// sth", /* token stream: */ "SL_COMMENT");
    
    testNotTerminal("/* sth * /", /* token stream: */ "ML_COMMENT");
    testNotTerminal("/ / sth", /* token stream: */ "SL_COMMENT");
  }

  @Test
  public void bigint(){
    testTerminal("123", /* token stream: */ "BIGINT");
    testTerminal("0", /* token stream: */ "BIGINT");
    testTerminal("1", /* token stream: */ "BIGINT");
    testTerminal("5", /* token stream: */ "BIGINT");
    testTerminal("9", /* token stream: */ "BIGINT");
    testTerminal("123456789012345678901234567890", /* token stream: */ "BIGINT");
    
    testNotTerminal("01", /* token stream: */ "BIGINT");
    testNotTerminal("00", /* token stream: */ "BIGINT");
    testNotTerminal("0a", /* token stream: */ "BIGINT");
    testNotTerminal("A", /* token stream: */ "BIGINT");
  }

  @Test
  public void string(){
    testTerminal("\"\\b\\t\\n\\f\\r\\\"\\'\\\\'\"", /* token stream: */ "STRING");
    testTerminal("\'ala ma kota\'", /* token stream: */ "STRING");
    testTerminal("\"ala ma kota\"", /* token stream: */ "STRING");
    
    testNotTerminal("\'ala ma kota\"", /* token stream: */ "STRING");
    testNotTerminal("\"ala ma kota\'", /* token stream: */ "STRING");
    testNotTerminal("\"aa", /* token stream: */ "STRING");
    testNotTerminal("aa\"", /* token stream: */ "STRING");
    testNotTerminal("\"0a'", /* token stream: */ "STRING");
    testNotTerminal("'A\"", /* token stream: */ "STRING");
  }

  @Test
  public void whitespace(){
    testTerminal(" ", /* token stream: */ "WS");
    testTerminal("\t", /* token stream: */ "WS");
    testTerminal("\r", /* token stream: */ "WS");
    testTerminal("\n", /* token stream: */ "WS");
    testTerminal("\n\t\r ", /* token stream: */ "WS");
    
    testNotTerminal("\\t", /* token stream: */ "WS");
    testNotTerminal("\\r", /* token stream: */ "WS");
    testNotTerminal("\\n", /* token stream: */ "WS");
    testNotTerminal("dfas fdsa", /* token stream: */ "WS");
  }
}
