package net.afyre.haha.demo.ui;

import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.registry.EditorDescriptor;
import org.eclipse.ui.internal.registry.EditorRegistry;
import org.eclipse.ui.internal.registry.FileEditorMapping;
import org.eclipse.xtext.ui.editor.XtextEditor;

@SuppressWarnings("restriction")
public class HahaEditor extends XtextEditor {

	private String[] unwantedContextMenuItems = {
			"QuickAssist",
			"Preferences.ContextAction",
			"settings",
			"additions",
			"group.find",
			"group.rest",
			"group.add",
			"group.edit",
			"group.print",
			"group.copy",
			//"group.open",// The HAHA context menu items are here 
			"group.save",
			"group.undo",
			"revert",
			//"undo", // These items are explicitly commented out to
			//"save", // mark that these items are present in the context
			//"cut",  // menu. These comments should not be erased.
			//"copy",
			//"paste"
	};

	/**
	 * The method is used to guarantee that only *.haha is supported in the
	 * General/Editors/File Associations page under the the Preferences menu item.
	 * 
	 * This method uses Eclipse functions that are not made open API so the
	 * code is likely to change with a change of the Eclipse version.
	 */
	@Override
	public void init(IEditorSite site, IEditorInput input) throws PartInitException { 
	
		IEditorRegistry eReg = PlatformUI.getWorkbench().getEditorRegistry();
		if (eReg instanceof EditorRegistry) {
			EditorRegistry editorReg = (EditorRegistry)eReg;
			IEditorDescriptor[] editors = editorReg.getEditors("*.haha", null);
			FileEditorMapping[] newMappings = new FileEditorMapping[1];
			FileEditorMapping mapping = new FileEditorMapping("haha");
			for (int i =0 ; i < editors.length; i++) {
				if (editors[i] instanceof EditorDescriptor) {
					mapping.addEditor((EditorDescriptor) editors[i]);
				}
			}
			newMappings[0]=mapping;
			editorReg.setFileEditorMappings(newMappings);
		}
	
		super.init(site, input);
	}
	
	/**
	 * This method is used to remove unwanted context menu items.
	 */
	@Override
	public void editorContextMenuAboutToShow(IMenuManager rmenu) {
		MenuManager menu = (MenuManager)rmenu;
		super.editorContextMenuAboutToShow(menu);
	    for (String itm : unwantedContextMenuItems) {
			menu.remove(itm);
		}
	    for (IContributionItem itm: menu.getItems()) {
	    	if (itm.getId()==null) {
	    		menu.remove(itm);
	    	}
	    }
	}
}
