package net.afyre.haha.demo.ui.annotations;

import net.afyre.haha.demo.solver.smt.SmtModel;
import net.afyre.haha.demo.vcgen.VerificationCondition;

/**
 * Annotation for assertions which were proven to be invalid.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class InvalidAssertAnnotation extends ConditionAnnotation {
  /** Annotation type. */
  public static final String ID = "net.afyre.haha.demo.ui.invalidassert";

  /** Counterexample (might be null). */
  private final SmtModel model;
  
  /**
   * Constructor.
   * @param condition The condition that caused the problem.
   * @param model Counterexample, {@code null} if not available.
   */
  public InvalidAssertAnnotation(final VerificationCondition condition, 
                                 final SmtModel model) {
    super(ID, condition);
    this.model = model;
  }
  
  /**
   * Return text created from VC context and model.
   * @return Text.
   */
  @Override
  public String getText() {
    String msg = "Verification condition is not valid.\n";
    msg += getVCDescription();
    if (model != null) {
      msg = msg + "\nCounterexample: \n  " + 
              model.toString().replace("\n", "\n  ");
    }
    return msg;
  }
  
  /**
   * Get the counterexample.
   * @return Counterexample model or {@code null}.
   */
  public SmtModel getModel() {
    return model;
  }
  
  /**
   * Check if a counterexample is available.
   * @return True if a model is available, false otherwise.
   */
  public boolean hasModel() {
    return model != null;
  }
}
