package net.afyre.haha.demo.ui.hover;

import net.afyre.haha.demo.solver.smt.SmtModel;
import net.afyre.haha.demo.vcgen.VerificationCondition;

/**
 * Data object used to pass information to markers.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class HahaMarkerInformation {
  /** Annotated verification condition. */
  private final VerificationCondition condition;
  
  /** Was the prover unable to prove/disprove the condition? */
  private final boolean unknown;
  
  /** Counterexample found by the solver (or null). */
  private final SmtModel model;
  
  /**
   * Constructor.
   * @param condition Verification condition associated with the annotation.
   * @param unknown Is the condition;s status UNKNOWN (as opposed to INVALID)?
   * @param model Counterexample found by the solver, null if not available.
   */
  public HahaMarkerInformation(final VerificationCondition condition, 
                               final boolean unknown,
                               final SmtModel model) {
    this.condition = condition;
    this.unknown = unknown;
    this.model = model;
  }

  /**
   * Get the associated verification condition.
   * @return Verification condition.
   */
  public VerificationCondition getCondition() {
    return condition;
  }

  /**
   * Check if the prover was unable to prove/disprove the condition?
   * @return True if the condition's status is UNKNOWN, 
   *         false if it is INVALID.
   */
  public boolean isUnknown() {
    return unknown;
  }

  /**
   * Get the counterexample provided by the solver.
   * @return Counterexample, or null if not available.
   */
  public SmtModel getModel() {
    return model;
  }
}
