package net.afyre.haha.demo.ui.hover;

import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.text.IRegion;
import org.eclipse.jface.text.ITextViewer;
import org.eclipse.jface.text.Position;
import org.eclipse.jface.text.Region;
import org.eclipse.jface.text.source.Annotation;
import org.eclipse.jface.text.source.IAnnotationHover;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.xtext.ui.editor.hover.AbstractProblemHover;

import com.google.common.collect.Sets;
import com.ibm.icu.text.BreakIterator;

/**
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class HahaTextHover extends AbstractProblemHover implements IAnnotationHover {
  private static final int TOOLTIP_WIDTH = 100;
  
  /** {@inheritDoc} */
  @Override
  public String getHoverInfo(ISourceViewer sourceViewer, int lineNumber) {
    Object o = getHoverInfoInternal(sourceViewer, lineNumber, -1);
    if (o != null)
        return o.toString();
    return null;
  }

  /** {@inheritDoc} */
  @Override
  protected IRegion getHoverRegionInternal(int lineNumber, int offset) {
    List<Annotation> annotations = getAnnotations(lineNumber, offset);
    for (Annotation annotation : annotations) {
        Position position = sourceViewer.getAnnotationModel().getPosition(annotation);
        if (position != null) {
            final int start = position.getOffset();
            return new Region (start, position.getLength());    
        }
    }
    return null;
  }

  /** {@inheritDoc} */
  @Override
  protected Object getHoverInfoInternal(ITextViewer textViewer, int lineNumber,
      int offset) {
    final Set<String> messages = Sets.newLinkedHashSet();
    List<Annotation> annotations = getAnnotations(lineNumber, offset);
    for (Annotation annotation : annotations) {
        if (annotation.getText() != null) {
            messages.add(annotation.getText().trim());
        }
    }
    if (messages.size() > 0) {
        return formatMessages(messages);
    }
    return null;
  }
  
  private String formatMessages(final Collection<String> messages) {
    final StringBuilder result = new StringBuilder();
    
    for (final String text : messages) {
      result.append("- ");
      boolean first = true;
      for (final String line : text.split("\n")) {
        if (!first) { result.append("  "); }
        first = false;
        
        // Uggghhh.
        String indent = getIndent(line);
        result.append(indent);
        indent = "  " + indent;
        final int indentLength = indent.length();
        
        final BreakIterator splitter = BreakIterator.getWordInstance();
        splitter.setText(line);
        int column = 2;
        for (int start = splitter.first(), end=splitter.next(); 
             end != BreakIterator.DONE;
             start = end, end = splitter.next()) {
          if (column + end - start > TOOLTIP_WIDTH && 
              column > indentLength) {
            column = indent.length();
            result.append("\n");
            result.append(indent);
          }
          result.append(line, start, end);
          column += end - start;
        }
        result.append("\n");        
      }
    }
    result.deleteCharAt(result.length() - 1);
    return result.toString();
  }
  
  private final String getIndent(final String line) {
    final StringBuilder result = new StringBuilder();
    
    int c;
    for (int pos = 0; 
         pos < line.length(); 
         pos += Character.charCount(c)) {
       c = line.codePointAt(pos);
       if (Character.isWhitespace(c)) {
         result.appendCodePoint(c);
       } else {
         break;
       }
     }
     return result.toString();
  }
}
