package net.afyre.haha.demo.ui.prefs;

import java.net.URL;

import net.afyre.haha.demo.ui.internal.HahaActivator;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.preference.DirectoryFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

/**
 * Preference page with Coq settings.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * @author Aleksy Schubert <alx@mimuw.edu.pl>
 *
 */
public class CoqPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage  {
  /** Coq file path preference name. */
  public static final String PREF_COQ_FILE_PATH = "COQ_FILE_PATH";
  public static final String PREF_COQ_FILE_PATH_DEFAULT = "tmp";
  public static final String PREF_COQ_FILE_PATH_DEFAULT_WIN = "Temp";
  public static final String PREF_COQ_SOLVER_PATH = "COQ_SOLVER_PATH";
  public static final String PREF_COQ_SOLVER_PATH_DEFAULT = "/usr/bin/coqc";
  public static final String PREF_COQ_SOLVER_PATH_DEFAULT_WIN = "C:\\Coq\\bin\\coqc.exe";
  public static final String PREF_COQ_IDE_PATH = "COQ_IDE_PATH";
  public static final String PREF_COQ_IDE_PATH_DEFAULT = "/usr/bin/coqide";
  public static final String PREF_COQ_IDE_PATH_DEFAULT_WIN = "C:\\Coq\\bin\\coqide.exe";
  public static final String PREF_COQ_CHECKER_PATH = "COQ_CHECKER_PATH";
  public static final String PREF_COQ_CHECKER_PATH_DEFAULT = "/usr/bin/coqchk";
  public static final String PREF_COQ_CHECKER_PATH_DEFAULT_WIN = "C:\\Coq\\bin\\coqchk.exe";

  public CoqPreferencePage() {
    super("Coq Preferences", FieldEditorPreferencePage.FLAT);
    final URL url = 
        FileLocator.find(HahaActivator.getInstance().getBundle(), 
                         new Path("icons/coq.gif"), null);
    final ImageDescriptor image = ImageDescriptor.createFromURL(url);
    setImageDescriptor(image);
  }
  
  /** {@inheritDoc} */
  @Override
  protected void createFieldEditors() {
    final Composite parent = getFieldEditorParent();

    addField(new DirectoryFieldEditor(PREF_COQ_FILE_PATH, "Coq file path:", parent));
    addField(new ExecutableFieldEditor(PREF_COQ_SOLVER_PATH, "Coq compiler path:", parent));
    addField(new ExecutableFieldEditor(PREF_COQ_IDE_PATH, "Coq IDE path:", parent));
    addField(new ExecutableFieldEditor(PREF_COQ_CHECKER_PATH, "Coq checker path:", parent));
  }
  
  /** {@inheritDoc} */
  @Override
  public void init(IWorkbench workbench) {
    setPreferenceStore(HahaActivator.getInstance().getPreferenceStore());
  }
  
}
