package net.afyre.haha.demo.ui.prefs;

import java.io.File;

import org.eclipse.jface.preference.StringButtonFieldEditor;
import org.eclipse.swt.widgets.Composite;

import net.afyre.haha.demo.util.FileFinder;

/**
 * Hacked version of FileFiledEditor.
 * 
 * TODO: Comment, cleanup.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * 
 */
public class ExecutableFieldEditor extends StringButtonFieldEditor {
  private final FileFinder fileFinder;


  /**
   * The method parses the file name and finds out possible paths where
   * the executable may be located as well as possible executable file
   * extensions.
   * 
   * @param name      the name of the executable file to be found
   * @param labelText the label of the button to show
   * @param parent    the parent of the current window
   */
  public ExecutableFieldEditor(final String name, final String labelText,
      final Composite parent) {
    super(name, labelText, parent);

    fileFinder = new FileFinder();
  }



  protected String changePressed() {
    File f = new File(getTextControl().getText());
    if (!f.exists()) {
      f = null;
    }
    File d = getFile(f);
    if (d == null) {
      return null;
    }

    return d.getAbsolutePath();
  }

  protected boolean checkState() {

    String msg = null;

    String path = getTextControl().getText();
    if (path != null) {
      path = path.trim();
    } else {
      path = "";//$NON-NLS-1$
    }
    if (path.length() == 0) {
      if (!isEmptyStringAllowed()) {
        msg = getErrorMessage();
      }
    } else {
      File file = new File(path);
      if (file.getParent() == null) {
        file = fileFinder.findFileOnPath(path);
      }
      if (!file.exists()) {
        msg = "File does not exist";
      } else if (!file.canExecute()) {
        msg = "File is not executable.";
      }
    }

    if (msg != null) { // error
      showErrorMessage(msg);
      return false;
    }

    if (doCheckState()) {
      clearErrorMessage();
      return true;
    }
    msg = getErrorMessage();
    if (msg != null) {
      showErrorMessage(msg);
    }
    return false;
  }

  
  private File getFile(File startingDirectory) {
    String file = fileFinder.getFileThroughDialog(startingDirectory, getShell());
    if (file != null) {
      file = file.trim();
      if (file.length() > 0) {
        return new File(file);
      }
    }
    return null;
  }




  

}
