package net.afyre.haha.demo.ui.prefs;

import net.afyre.haha.demo.ui.internal.HahaActivator;

import java.io.File;

import org.eclipse.core.runtime.preferences.AbstractPreferenceInitializer;
import org.eclipse.jface.preference.IPreferenceStore;

/**
 * Initializer for all preference pages.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class HahaPreferenceInitializer extends AbstractPreferenceInitializer {


/** {@inheritDoc} */
  @Override
  public void initializeDefaultPreferences() {
	final String home = getHome();
	final IPreferenceStore store = 
	        HahaActivator.getInstance().getPreferenceStore();
    store.setDefault(SolverPreferencePage.PREF_SOLVER_VERSION, 
    SolverPreferencePage.PREF_SOLVER_VERSION_DEFAULT);
		if (isWin()) {
		    store.setDefault(SolverPreferencePage.PREF_SOLVER_PATH, 
			         SolverPreferencePage.PREF_SOLVER_PATH_DEFAULT_WIN);
			store.setDefault(CoqPreferencePage.PREF_COQ_FILE_PATH, 
					home + CoqPreferencePage.PREF_COQ_FILE_PATH_DEFAULT_WIN);
			store.setDefault(CoqPreferencePage.PREF_COQ_SOLVER_PATH,
					CoqPreferencePage.PREF_COQ_SOLVER_PATH_DEFAULT_WIN);
			store.setDefault(CoqPreferencePage.PREF_COQ_IDE_PATH, 
					CoqPreferencePage.PREF_COQ_IDE_PATH_DEFAULT_WIN);
			store.setDefault(CoqPreferencePage.PREF_COQ_CHECKER_PATH,
					CoqPreferencePage.PREF_COQ_CHECKER_PATH_DEFAULT_WIN);
		} else {
		    store.setDefault(SolverPreferencePage.PREF_SOLVER_PATH, 
			         SolverPreferencePage.PREF_SOLVER_PATH_DEFAULT);
			store.setDefault(CoqPreferencePage.PREF_COQ_FILE_PATH, 
					home + CoqPreferencePage.PREF_COQ_FILE_PATH_DEFAULT);
			store.setDefault(CoqPreferencePage.PREF_COQ_SOLVER_PATH, 
					CoqPreferencePage.PREF_COQ_SOLVER_PATH_DEFAULT);
			store.setDefault(CoqPreferencePage.PREF_COQ_IDE_PATH, 
					CoqPreferencePage.PREF_COQ_IDE_PATH_DEFAULT);
			store.setDefault(CoqPreferencePage.PREF_COQ_CHECKER_PATH, 
					CoqPreferencePage.PREF_COQ_CHECKER_PATH_DEFAULT);
		}
  }

  /**
   * The method checks if the current system is Windows.
   * 
   * @return true when the current operating system is Windows
   */
private boolean isWin() {
	return System.getProperty("os.name").toLowerCase().contains("win");
	}

/**
 * The method returns the string with the home directory of the current user.
 * 
 * @return the string with the home directory ending with 
 */
private String getHome() {
	return System.getProperty("user.home") + File.separatorChar;
}
}
