package net.afyre.haha.demo.util.tests;

import static net.afyre.haha.demo.util.tests.Util.assertIteratorReturns;
import static org.junit.Assert.assertNotNull;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

import net.afyre.haha.demo.util.ConsIterable;

import org.junit.Test;

/**
 * Tests for the {@link ConsIterable} class.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 */
public class ConsIterableTest {

  /** GetInstance does not return null. */
  @Test
  public void testInstanceNotNull() {
    final Integer[] array = { 1, 2, 3, 4 };
    final Iterable<Integer> tail = Arrays.asList(array);
    final Iterable<Integer> iterable = 
        ConsIterable.getInstance(0, tail);
    assertNotNull("Null returned from getInstance", iterable);
  }

  /** NPE is thrown for null tail. */
  @Test(expected=NullPointerException.class)
  public void testNullSource() {
    ConsIterable.getInstance(1, null);
  }
  
  /** Returned iterator is not null. */
  @Test
  public void testIteratorNotNull() {
    final Integer[] array = { 1, 2, 3, 4 };
    final Iterable<Integer> tail = Arrays.asList(array);
    final Iterable<Integer> iterable = 
        ConsIterable.getInstance(0, tail);
    final Iterator<Integer> it = iterable.iterator();
    assertNotNull("Null iterator returned", it);
  }
  
  /** Iteration works for simple sequences. */
  @Test
  public void testSimpleIteration() {
    final Integer[] array = { 1, 2, 3, 4 };
    final Iterable<Integer> tail = Arrays.asList(array);
    final Iterable<Integer> iterable = 
        ConsIterable.getInstance(0, tail);
    final Iterator<Integer> it = iterable.iterator();
    assertIteratorReturns(it, 0, 1, 2, 3, 4);
  }
  
  /** Iterator works correctly for empty tail. */
  @Test
  public void testEmpty() {
    final Integer[] array = {  };
    final Iterable<Integer> tail = Arrays.asList(array);
    final Iterable<Integer> iterable = 
        ConsIterable.getInstance(42, tail);
    final Iterator<Integer> it = iterable.iterator();
    assertIteratorReturns(it, 42);
  }
  
  /** Returned iterator does not support remove(). */
  @Test(expected=UnsupportedOperationException.class)
  public void testRemove() {
    final ArrayList<Integer> tail = new ArrayList<Integer>();
    tail.add(1);
    tail.add(2);
    tail.add(3);
    final Iterable<Integer> iterable = 
        ConsIterable.getInstance(0, tail);
    final Iterator<Integer> it = iterable.iterator();
    it.remove();
  }
}
