/**
 * 
 */
package net.afyre.haha.demo.util.tests;

import static org.junit.Assert.*;
import static org.junit.Assume.*;

import net.afyre.haha.demo.util.FNVBuilder;

import org.junit.Test;

/**
 * Tests for the {@link FNVBuilder} class.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class FNVBuilderTest {
  /** GetInstance does not return null. */
  @Test
  public void testInstanceNotNull() {
    final FNVBuilder builder = FNVBuilder.getInstance();
    assertNotNull("Null returned from getInstance", builder);
  }
  
  /** 
   * Hash after appendInt is equal to that returned 
   * after appending four bytes individually.
   */
  @Test
  public void testAppendInt() {
    final FNVBuilder builder1 = FNVBuilder.getInstance();
    assumeNotNull("Null returned from getInstance", builder1);
    builder1.appendInt(0x12345678);
    final FNVBuilder builder2 = FNVBuilder.getInstance();
    assumeNotNull("Null returned from getInstance", builder2);
    builder2.appendByte((byte)0x78);
    builder2.appendByte((byte)0x56);
    builder2.appendByte((byte)0x34);
    builder2.appendByte((byte)0x12);
    assertEquals("Wrong bytes appended in appendInt", 
                 builder2.getHash(), builder1.getHash());
  }
  
  /** Hash after reset equals hash of an empty sequence. */
  @Test
  public void testReset() {
    final FNVBuilder builder = FNVBuilder.getInstance();
    assumeNotNull("Null returned from getInstance", builder);
    final int emptyHash = builder.getHash();
    builder.appendByte((byte)0x78);
    builder.appendByte((byte)0x56);
    builder.appendByte((byte)0x34);
    builder.appendByte((byte)0x12);
    final int otherHash = builder.getHash();
    assertFalse("Hash function too weak", emptyHash == otherHash);
    builder.reset();
    final int resetHash = builder.getHash();
    assertEquals("Wrong hash after reset", emptyHash, resetHash);
  }
}
