package net.afyre.haha.demo.util.tests;

import static org.junit.Assert.*;
import static net.afyre.haha.demo.util.tests.Util.*;

import java.util.ArrayList;

import net.afyre.haha.demo.util.SuperclassIterator;

import org.junit.Test;

/**
 * Tests for the {@link SuperclassIterator} class.
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public class SuperclassIteratorTest {
  /** GetInstance does not return null. */
  @Test
  public void testInstanceNotNull() {
    final ArrayList<Integer> array = new ArrayList<Integer>();
    array.add(1);
    array.add(2);
    array.add(3);
    array.add(4);
    final SuperclassIterator<Object> it = 
        SuperclassIterator.<Object>getInstance(array.iterator());
    assertNotNull("Null returned from getInstance()", it);
  }
  
  /** NPE is thrown if null is passed to getInstance(). */
  @Test(expected=NullPointerException.class)
  public void testGetInstanceWithNull1() {
    SuperclassIterator.getInstance(null);
  }
  
  /** Wrapped iterator returns correct elements. */
  @Test
  public void testIterator() {
    final ArrayList<Integer> array = new ArrayList<Integer>();
    array.add(1);
    array.add(2);
    array.add(3);
    array.add(4);
    final SuperclassIterator<Object> it = 
        SuperclassIterator.<Object>getInstance(array.iterator());
    assertIteratorReturns(it, 1, 2, 3, 4);
  }
  
  /** Wrapped iterator forwards remove() requests. */
  @Test
  public void testRemove() {
    final ArrayList<Integer> array = new ArrayList<Integer>();
    array.add(1);
    array.add(2);
    array.add(3);
    array.add(4);
    final SuperclassIterator<Object> it = 
        SuperclassIterator.<Object>getInstance(array.iterator());
    it.next();
    it.next();
    it.next();
    it.remove();
    assertArrayEquals("Wrong array state after remove()", 
                      new Object[] { 1, 2, /* 3, */ 4}, 
                      array.toArray(new Object[]{}));
  }
}
