package net.afyre.haha.demo.util;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * Iterator over an empty collection.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * 
 * @param <T> Type of elements.
 */
public final class EmptyIterator<T> implements Iterator<T> {
  /** Constructor. */
  private EmptyIterator() { }
  
  /**
   * Create an empty iterator.
   * @param <T> Type of elements.
   * @return Iterator instance.
   */
  public static<T> EmptyIterator<T> getInstance() {
    return new EmptyIterator<T>();
  }
  
  /** {@inheritDoc} */
  @Override
  public boolean hasNext() {
    return false;
  }

  /** {@inheritDoc} */
  @Override
  public T next() {
    throw new NoSuchElementException();
  }

  /** {@inheritDoc} */
  @Override
  public void remove() {
    throw new IllegalStateException();
  }

}
