package net.afyre.haha.demo.util;

import java.io.File;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;

public class FileFinder {

	private File[] executablePath;
	private String[] executableExtensions;

	public FileFinder() {
		final String systemPath = System.getenv("PATH");
		if (systemPath == null) {
			executablePath = new File[0];
		} else {
			final String[] pathDirs = systemPath.split(File.pathSeparator);
			executablePath = new File[pathDirs.length];
			for (int i = 0; i < pathDirs.length; i++) {
				executablePath[i] = new File(pathDirs[i]);
			}
		}
		final String systemPathExt = System.getenv("PATHEXT");
		if (systemPathExt == null) {
			executableExtensions = new String[0];
		} else {
			final String[] exts = systemPathExt.split(File.pathSeparator);
			executableExtensions = exts;
		}
	}

	public File findFileOnPath(final String file) {
		for (final File dir : executablePath) {
			for (final String ext : executableExtensions) {
				final File exe = new File(dir, file + ext);
				if (exe.exists()) {
					return exe;
				}
			}
			final File exe = new File(dir, file);
			if (exe.exists()) {
				return exe;
			}
		}

		return new File(file);
	}
	
	public String getFileThroughDialog(File startingDirectory, Shell shell) {
		final FileDialog dialog = new FileDialog(shell, SWT.OPEN | SWT.SHEET);
	    if (startingDirectory != null) {
	      dialog.setFileName(startingDirectory.getPath());
	    }
	    if (executableExtensions.length > 0) {
	      final String[] extensions = new String[executableExtensions.length];
	      for (int i = 0; i < executableExtensions.length; i++) {
	        extensions[i] = "*" + executableExtensions[i];
	      }
	      dialog.setFilterExtensions(extensions);
	      dialog.setFilterIndex(getFilterIndex(executableExtensions));
	    }
	    String file = dialog.open();
		return file;
	}

	  /**
	   * Get index of the initial filter used to select files.
	   * 
	   * @param extensions List of executable extensions.
	   * @return Index of '.exe' or 0.
	   */
	  private int getFilterIndex(final String[] extensions) {
	    for (int i = 0; i < extensions.length; i++) {
	      if (extensions[i].equalsIgnoreCase(".exe")) {
	        return i;
	      }
	    }
	    return 0;
	  }
}
