package net.afyre.haha.demo.util;

/**
 * Utility functions related to hashing.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 */
public final class HashUtil {
  /** Singleton instance. */
  private static final HashUtil instance = new HashUtil();
    
  /** Constructor. */
  private HashUtil() { }
  
  /**
   * Get an instance of HashUtil.
   * @return Instance.
   */
  public static HashUtil getInstance() {
    return instance;
  }

  /**
   * Create an instance of default hash builder.
   * @return Hash builder.
   */
  public HashBuilder getHashBuilder() {
    return FNVBuilder.getInstance();
  }
  
  /**
   * Calculate hash of a byte array.
   * @param bytes Input data.
   * @return Hash value.
   */
  public int hashBytes(final byte[] bytes) {
    final FNVBuilder builder = FNVBuilder.getInstance();
    for (final byte b : bytes) {
      builder.appendByte(b);
    }
    return builder.getHash();
  }
  
  /**
   * Calculate hash value for a sequence of objects.
   * 
   * @param objects Objects. Null values are permitted.
   * @return Composed hash value.
   */
  public int hash(final Object... objects) {
    final FNVBuilder builder = FNVBuilder.getInstance();
    for (final Object obj : objects) {
      if (obj == null) {
        builder.appendInt(0);
      } else {
        builder.appendInt(obj.hashCode());
      }
    }
    return builder.getHash();    
  }
  
}
