package net.afyre.haha.demo.util;

/**
 * Immutable pair of elements.
 *
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 * @param <L> Type of the first component.
 * @param <R> Type of the second component.
 */
public class Pair<L, R> {
  /** Object used to compute hash values. */
  protected static final HashUtil hashUtil = HashUtil.getInstance();
  
  /** Object used for equality testing. */
  protected static final CmpUtil cmpUtil = CmpUtil.getInstance();
  
  /** First element. */
  protected final L first;
  
  /** Second element. */
  protected final R second;
  
  /** Cached hash code. */
  private final int hash;
  
  /**
   * Constructor (internal use only).
   * @param first First element.
   * @param second Second element.
   */
  protected Pair(final L first, final R second) {
    this.first = first;
    this.second = second;
    this.hash = hashUtil.hash(first, second);
  }
  
  /**
   * Create a new pair.
   * @param first First element.
   * @param second Second element.
   * @return Pair instance.
   */
  public static <L, R> Pair<L, R> 
  getInstance(final L first, final R second) {
    return new Pair<L, R>(first, second);
  }
  
  /**
   * Get the first element.
   * @return First element.
   */
  public final L getFirst() {
    return first;
  }
  
  /**
   * Get the second element.
   * @return Second element.
   */
  public final R getSecond() {
    return second;
  }
  
  /** {@inheritDoc} */
  @Override
  public final boolean equals(final Object other) {
    if (other instanceof Pair) {
      final Pair<?, ?> p = (Pair<?, ?>)other;
      return 
          cmpUtil.safeEquals(first, p.first) &&
          cmpUtil.safeEquals(second, p.second);
    } else {
      return false;
    }
  }
  
  /** {@inheritDoc} */
  @Override
  public final int hashCode() {
    return hash;
  }
}
