/**
 * 
 */
package net.afyre.haha.demo.util;

import java.util.Iterator;

/**
 * Trivial iterator wrapper used to turn an
 * iterator over a subclass into an iterator
 * over a superclass.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * 
 * @param <T> type of elements returned by this iterator.
 */
public final class SuperclassIterator<T> implements Iterator<T> {
  /** WRapped iterator. */
  private final Iterator<? extends T> source;
  
  /**
   * Constructor.
   * @param source Source iterator.
   * @throws NullPointerException of the source iterator is null.
   */
  private SuperclassIterator(final Iterator<? extends T> source) {
    if (source == null) {
      throw new NullPointerException("source");
    }
    this.source = source;
  }
  
  /**
   * Wrap an iterator.
   * @param <T> type of returned elements.
   * @param source Source iterator.
   * @result Wrapped iterator.
   * @throws NullPointerException of the source iterator is null.
   */
  public static<T> SuperclassIterator<T> getInstance(
      final Iterator<? extends T> source) {
    return new SuperclassIterator<T>(source);
  }
  
  /** {@inheritDoc} */
  @Override
  public boolean hasNext() {
    return source.hasNext();
  }

  /** {@inheritDoc} */
  @Override
  public T next() {
    return source.next();
  }

  /** {@inheritDoc} */
  @Override
  public void remove() {
    source.remove();
  }

}
