package net.afyre.haha.demo.util;

import java.util.Iterator;

/**
 * Combines two iterators into an iterator over pairs.. 
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 * 
 * @param <L> Elements of the first iterator.
 * @param <R> Elements of the second iterator.
 */
public final class ZipIterator<L, R> implements Iterator<Pair<L, R>> {
  /** First iterator. */
  private final Iterator<? extends L> left;
  
  /** Second iterator. */
  private final Iterator<? extends R> right;
  
  /**
   * Constructor.
   * @param left First iterator.
   * @param right Second iterator.
   * @throws NullPointerException If given a null argument.
   */
  private ZipIterator(final Iterator<? extends L> left, 
                      final Iterator<? extends R> right) {
    if (left == null) { throw new NullPointerException("left"); }
    if (right == null) { throw new NullPointerException("right"); }
    this.left = left;
    this.right = right;
  }

  /**
   * Combine two iterators into an iterator over pairs.
   * @param <L> Elements of the first iterator.
   * @param <R> Elements of the second iterator.
   * @param left First iterator.
   * @param right Second iterator.
   * @return New iterator.
   * @throws NullPointerException If given a null argument.
   */
  public static<L, R> Iterator<Pair<L, R>> getInstance(
      final Iterator<? extends L> left, 
      final Iterator<? extends R> right) {
    return new ZipIterator<L, R>(left, right);
  }
  
  /** {@inheritDoc} */
  @Override
  public boolean hasNext() {
    return left.hasNext() && right.hasNext();
  }

  /** {@inheritDoc} */
  @Override
  public Pair<L, R> next() {
    return Pair.<L, R>getInstance(left.next(), right.next());
  }

  /** {@inheritDoc} */
  @Override
  public void remove() {
    throw new UnsupportedOperationException();
  }
  
  
}
