package net.afyre.haha.demo.util.async;

import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

/**
 * A computation that returns a constant value without blocking.
 * 
 * @author Tadeusz Sznuk <tsznuk@mimuw.edu.pl>
 *
 * @param <T> Type of the returned value.
 */
public final class ConstFuture<T> implements Future<T> {
  /** Precomputed value. */
  private final T value;
  
  /**
   * Create an instance of this computation.
   * @param <T> Type of the returned value.
   * @param value Value to be returned.
   * @return Instance.
   */
  public static<T> ConstFuture<T> getInstance(final T value) {
    return new ConstFuture<T>(value);
  }
  
  /**
   * Constructor.
   * @param value Value to be returned.
   */
  protected ConstFuture(final T value) {
    this.value = value;
  }
  
  /** {@inheritDoc} */
  @Override
  public boolean cancel(final boolean mayInterruptIfRunning) {
    return false;
  }

  /** {@inheritDoc} */
  @Override
  public T get() throws InterruptedException, ExecutionException {
    return value;
  }

  /** {@inheritDoc} */
  @Override
  public T get(final long timeout, final TimeUnit unit) 
    throws InterruptedException, ExecutionException, TimeoutException {
    return value;
  }

  /** {@inheritDoc} */
  @Override
  public boolean isCancelled() {
    return false;
  }

  /** {@inheritDoc} */
  @Override
  public boolean isDone() {
    return true;
  } 
}